Attribute VB_Name = "modFracZn2C"
'===========================================================
' Descrizione.....: Routines per la sequenza: Zn+1 = Zn + C
' Nome dei Files..: modFracZn2C.bas
' Data............: 29/4/1989 (QBasic)
' Revisione.......: 11/2/2001
' Versione........: 1.0 a 32 bits.
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'===========================================================
'
'   Teoria e viste della sequenza Zn+1 = Zn + C
'   sono state ricavate da:
'    The Beauty of Fractals
'    di H. O. Peitgen, P. H. Richter
'    Springer-Verlag
'
Option Explicit
'
Public X1P As Long      ' Estremi fisici della vista
Public X2P As Long      ' dati dalle dimensioni
Public Y1P As Long      ' del PictureBox.
Public Y2P As Long      '
'
Private xir As Double   ' Variabili per le
Private dxr As Double   ' trasformazioni da
Private dyr As Double   ' coordinate fisiche
Private yir As Double   ' a coordinate reali.
'
' Per 2 colori:
Public Const NT0Col& = 2        ' N di colori.
Public T0Col&(0 To NT0Col - 1)  ' Tabella dei colori.
'
' Per 16 colori:
Public Const NT1Col& = 16       ' N di colori.
Public T1Col&(0 To NT1Col - 1)  ' Tabella dei colori.
'
' Per 256 colori:
Public Const NT2Col& = 256      ' N di colori.
Public T2Col&(0 To NT2Col - 1)  ' Tabella dei colori.
'
' Per 1280 colori:
Public Const NT3Col& = 5 * 256  ' N di colori.
Public T3Col&(0 To NT3Col - 1)  ' Tabella dei colori.
'
Private TCol&() ' Tabella dei colori in uso.
'
' N massimo di iterazioni della sequenza:
Private MAXLOOP&
'
' Soglia dell' attrattore all' infinito:
Private Const MaxSize# = 10000#
'
Private Declare Function SetPixelV Lib "gdi32" (ByVal hDC As Long _
, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long

Public Sub Mandelbrot(ByVal Foglio As PictureBox _
    , ByVal X1R#, ByVal Y1R#, ByVal X2R#, ByVal Y2R# _
    , ByVal NTCol&, ByVal K&, ByRef tElab!, ByRef bStop As Boolean)
'
'
    Dim I&, X&, Y&, PColor&
    Dim Zr#, Zi#, Zr2#, Zi2#, Cr#, Ci#
    
'
    Impostazioni X1R, Y1R, X2R, Y2R, NTCol, K
'
    tElab = Timer
'
    ' Esplora la vista pixel per pixel:
    For Y = Y1P To Y2P      ' Esplora le righe.
        Ci = yMap(Y)        ' Calcola la parte immaginaria
                            ' di C corrispondente alla riga.
'
        For X = X1P To X2P  ' Esplora le colonne.
            Cr = xMap(X)    ' Calcola la parte reale
                            ' di C corrispondente alla colonna.
'
            ' Calcola la posizione del punto
            ' rispetto all' insieme di Mandelbrot:
            Zr = 0#
            Zi = 0#
            I = 0
            Do
                If I = MAXLOOP Then
                    PColor = vbBlack
                    Exit Do
                End If
'
                Zr2 = Zr * Zr
                Zi2 = Zi * Zi
                If (Zr2 + Zi2) > MaxSize Then
                    PColor = TCol(I Mod NTCol)
                    Exit Do
                End If
'
                ' Per ottenere lo "Scarafone"
                ' invertire l' ordine delle due
                ' istruzioni seguenti:
                Zi = 2# * Zr * Zi + Ci
                Zr = Zr2 - Zi2 + Cr
                I = I + 1
            Loop
'
            ' Assegna un colore al punto:
            SetPixelV Foglio.hDC, X, Y, PColor
'
        Next X
'
        DoEvents
        If bStop Then Exit For
        Foglio.Refresh
    Next Y
'
    tElab = Timer - tElab
'
'
'
End Sub
Public Sub JuliaSet(ByVal Foglio As PictureBox, ByVal Cr#, ByVal Ci# _
    , ByVal X1R#, ByVal Y1R#, ByVal X2R#, ByVal Y2R# _
    , ByVal NTCol&, ByVal K&, ByRef tElab!, ByRef bStop As Boolean)
'
'
    Dim I&, X&, Y&, PColor&
    Dim Zr#, Zi#, Zi0#, Zr2#, Zi2#
    
'
    Impostazioni X1R, Y1R, X2R, Y2R, NTCol, K
'
    tElab = Timer
'
    ' Esplora la vista pixel per pixel:
    For Y = Y1P To Y2P      ' Esplora le righe.
        Zi0 = yMap(Y)
'
        For X = X1P To X2P  ' Esplora le colonne.
            ' Calcola la posizione del punto
            ' rispetto all' insieme di Julia:
            Zr = xMap(X)   ' Valore iniziale della parte reale di Z.
            Zi = Zi0       ' Valore iniziale della parte immaginaria di Z.
            I = 0
            Do
                If I = MAXLOOP Then
                    PColor = vbBlack
                    Exit Do
                End If
'
                Zr2 = Zr * Zr
                Zi2 = Zi * Zi
                If (Zr2 + Zi2) > MaxSize Then
                    PColor = TCol(I Mod NTCol)
                    Exit Do
                End If
'
                Zi = 2# * Zr * Zi + Ci
                Zr = Zr2 - Zi2 + Cr
                I = I + 1
            Loop
'
            ' Assegna un colore al punto:
            SetPixelV Foglio.hDC, X, Y, PColor
'
        Next X
        DoEvents
        If bStop Then Exit For
        Foglio.Refresh
    Next Y
'
    tElab = Timer - tElab
'
'
'
End Sub

Public Sub Main()
'
'
    ' Pepara le tabelle
    ' dei colori:
    TabelleColori
'
    frmFracZn2C.Show
'
    ' Inizia con l' immagine
    ' dell' insieme di Mandelbrot:
    frmFracZn2C.cmdMandelbrot.SetFocus
    frmFracZn2C.cmdMandelbrot_Click
'
'
'
End Sub


Private Sub TabelleColori()
'
'
    Dim C&, C1&, C2&
    Dim R&, G&, B&
'
    ' Prepara la tabella a 2 colori:
    T0Col(0) = vbWhite
    T0Col(1) = &H808080 ' Grigio.
'
    ' Prepara la tabella a 16 colori:
    For C = 0 To 15
        'R = 17 * C
        'G = CLng(IIf(C < 8, 36.4 * C, 546 - 36.4 * C))
        'B = 255 - 17 * C
        R = 255 * (Sqr(C) / Sqr(15))
        If C < 8 Then
            G = CLng(255 * (Sqr(C) / Sqr(7)))
        Else
            G = CLng(255 * (Sqr(15 - C) / Sqr(7)))
        End If
        B = 255 * (1! - Sqr(C) / Sqr(15))
'
        T1Col(C) = RGB(R, G, B)
    Next C
'
    ' Prepara la tabella a 256 colori:
    For C = 0 To 255
        'R = C
        'G = IIf(C < 128, 2 * C, 510 - 2 * C)
        'B = 255 - C
        R = 255 * (Sqr(C) / Sqr(255))
        If C < 128 Then
            G = CLng(255 * (Sqr(C) / Sqr(127)))
        Else
            G = CLng(255 * (Sqr(255 - C) / Sqr(127)))
        End If
        B = 255 * (1! - Sqr(C) / Sqr(255))
'
        T2Col(C) = RGB(R, G, B)
    Next C
'
    ' Prepara la tabella a 1280 colori:
    C = 0
    For C1 = 0 To 4
        For C2 = 0 To 255
            R = Switch(C1 = 0, 255 - C2 _
                     , C1 = 1, 0 _
                     , C1 = 2, 0 _
                     , C1 = 3, C2 _
                     , C1 = 4, 255)
            G = Switch(C1 = 0, 0 _
                     , C1 = 1, C2 _
                     , C1 = 2, 255 _
                     , C1 = 3, 255 _
                     , C1 = 4, 255 - C2)
            B = Switch(C1 = 0, 255 _
                     , C1 = 1, 255 _
                     , C1 = 2, 255 - C2 _
                     , C1 = 3, 0 _
                     , C1 = 4, 0)
'
            T3Col(C) = RGB(R, G, B)
            C = C + 1
        Next C2
    Next C1
'
'
'
End Sub

Public Function yMap(ByVal Yf&) As Double
'
'   Ritorna il valore dell' ordinata reale
'   corrispondente all' ordinata fisica Yf
'   di un pixel:
'
    yMap = yir + CDbl(Yf - Y1P) * dyr
'
'
'
End Function
Public Function xMap(ByVal Xf&) As Double
'
'   Ritorna il valore dell' ascissa reale
'   corrispondente all' ascissa fisica Xf
'   di un pixel:
'
    xMap = xir + CDbl(Xf - X1P) * dxr
'
'
'
End Function


Private Sub Impostazioni(ByVal X1R#, ByVal Y1R#, ByVal X2R#, ByVal Y2R# _
    , ByVal NTCol&, ByVal K&)
'
'
    ' Imposta i parametri per la
    ' trasformazione di coordinate:
    xir = X1R
    dxr = (X2R - X1R) / CDbl(X2P - X1P)
    yir = Y2R
    dyr = (Y1R - Y2R) / CDbl(Y2P - Y1P)
'
    ' Sceglie la tabella dei colori
    ' da usare ed imposta il N massimo
    ' di iterazioni della sequenza:
    Select Case NTCol
        Case NT0Col
        MAXLOOP = K * NT0Col
        TCol() = T0Col()
'
        Case NT1Col
        MAXLOOP = K * NT1Col
        TCol() = T1Col()
'
        Case NT2Col
        MAXLOOP = K * NT2Col
        TCol() = T2Col()
'
        Case NT3Col
        MAXLOOP = K * NT3Col
        TCol() = T3Col()
    End Select
'
'
'
End Sub
